/*
 * cnice Version 0.6
 * cnice: or bnice - a program for machines that do not have many colors.
 * It needs a machine that has a colormap. It tries to find the closest 
 * matching color.
 * GNU copyright: Gaspar Sinai <gsinai@gol.com> 24 Oct 1996
 * GNU Copyright: Gaspar Sinai <gsinai@gol.com> 29 Oct 1996
 */
#ifndef __colormap_h
#define __colormap_h

#include <stdio.h>
#include <varargs.h>
#include <malloc.h>
#include <X11/Xlib.h>

/* The minimal number of colors */
#define ALLOC_LIMIT	20

/* The maximum distance to match 0-1000 */
#define DISTANCE	70

typedef enum { STYLE_ALLOC = 1} STYLE_TYPE;
typedef enum 
{
	APPROXIMATE_MATCH = 1, 
	EXACT_MATCH, 
	FAR_MATCH, 
	ALLOCATED_MATCH
} MATCH_TYPE;

/*
 * These flags indicate the availability of a 
 * particular color cell.
 */
typedef enum 
{
	FLAG_FREE = 0, 
	FLAG_SHARED, 
	FLAG_ISHARE, 
	FLAG_IALLOC, 
	FLAG_USED
} FLAG_TYPE;

typedef unsigned long Pixel;

typedef struct 
{
	Display		*display;
	Screen		*screen;
	int		cells;
	XColor		*colors;
	char		*flags;
	Colormap	defaultColormap;
	Visual		*visual;
	Pixel		*pixels;
	short		distance;
	STYLE_TYPE	style;
	int		allocLimit;
	int		free;
	int		used;
	int		ishare;
	int		ialloc;
	int		shared;
} colorMap;


extern int initColorMap (colorMap *map);
extern int getFreeEntries (colorMap *map);
extern MATCH_TYPE grabColor (colorMap *map, char *colorName, XColor *exact, XColor *outMine, XColor *outX);
extern int freeColorMap (colorMap *map);
extern int getUsage (colorMap *map);
extern int printColorMap (FILE *handle, colorMap *map, int verbose);

#endif /* __colormap_h */
