/*
 * cnice Version 0.6
 * This program is to show how many cells are allocated in the standard
 * colormap.
 * GNU Copyright Gaspar Sinai <gsinai@gol.com> 24 Oct 1996
 * GNU Copyright Gaspar Sinai <gsinai@gol.com> 29 Oct 1996
 */

#include "rdb.h"
#include <string.h>

extern int	warn();
/*
 * This indicated that the first two entries in the array can be regarded as 
 * background color.
 */
#define BACKGROUND_LIKE 2

static char *colorTypes[] =
{
	"background:",
	"Background:",
	"foreground:",
	"Foreground:",
	"color:",
	"Color:"
};

static void rgb2HSV (unsigned short r, unsigned short g, unsigned short b, double *hr, double *sr, double *vr);
static void substitute (colorMap *map, char *segment, int segmentSize, char *newResource, XColor *desiredColor);

#define LINE_SIZE 	4096
#define COLOR_SIZE 	64

/*------------------------------------------------------------------------------
 * Function     : modifyRDB
 * Description  : modify resource databse lines and print them out.
 *                If motif flag != 0 than atricficially create the motif 
 *                shading colors. (Thanks to lesstif for the algorithm: 
 *                www.hungry.com)
 *                Dont expect this routine to get what is not in the resource
 *                lines: it can only guess that you are talking about colors
 *                there...
 * Returns      : 1
 *----------------------------------------------------------------------------*/
int
modifyRDB (FILE *file, colorMap *map, int motif)
{
	char	line [LINE_SIZE];
	char	colorName[COLOR_SIZE];
	XColor	background;
	XColor	foreground;
	XColor	topShadowColor;
	XColor	bottomShadowColor;
	XColor	armColor;

	XColor	exact, use, xout;
	int	i;
	char	*resStart;
	int	colorStart;
	int	colorEnd;
	double  h,s,v;
	char	*motifName;
	int	isUpper;

	/*
	 * Lines will be truncated at LINE_SIZE. Sorry.
	 */
	while (fgets (line, LINE_SIZE-1, file))
	{
		line[LINE_SIZE-1] = 0;
	
		resStart=NULL;
		for (i=0; i<sizeof (colorTypes)/sizeof (char*); i++)
		{
			if ((resStart = strstr (line, colorTypes[i]))) break;;
		}
		if (resStart==NULL)
		{
			fputs (line, stdout);
			continue;
		}
		colorStart = (resStart-line) + strlen(colorTypes[i]);
		while (line[colorStart] == ' ' || line[colorStart] == '\t') 
			colorStart++;
		colorEnd = colorStart;

		/*
		 * color names can contain spaces
		 */
		while (    line[colorEnd] != '\\'
			&& line[colorEnd] != '\t'
			&& line[colorEnd] != '\n'
			&& line[colorEnd] != '\0' ) colorEnd++;
		while (line[colorEnd] == ' ' || line[colorEnd] == '\t') 
			colorEnd--;
	
		if (colorStart == colorEnd || colorEnd - colorStart > COLOR_SIZE - 1)
		{
			fputs (line, stdout);
			continue;
		}
		memcpy (colorName, line + colorStart, colorEnd-colorStart);
		colorName[colorEnd-colorStart] = 0;
	
		if (!grabColor (map, colorName, &exact, &use, &xout))
		{
			warn ("No color mathes %s\n", colorName);
			fputs (line, stdout);
			continue;
		}
		/* print modified resource */
		fwrite (line, colorStart, 1, stdout);
		fprintf (stdout, "#%04x%04x%04x", 
		(int)use.red,(int)use.green,(int)use.blue);
		fprintf (stdout, "%s", line + colorEnd);

		/* if it is the background */
		if (!motif || i >= BACKGROUND_LIKE)
		{
			continue;
		}
		background = use;

		/* we need this because of the v value */
		rgb2HSV (background.red, background.green, background.blue, &h,&s,&v);
		if (v<0.5)
		{
			foreground.red = 0xFFFF;
			foreground.green = 0xFFFF;
			foreground.blue = 0xFFFF;
		}
		else
		{
			foreground.red = 0;
			foreground.green = 0;
			foreground.blue = 0;
		}
		armColor.red = background.red * .7;
		armColor.green = background.green * .7;
		armColor.blue = background.blue * .7;

		topShadowColor.red = (background.red > 65535/1.5) ?
			65535 : background.red * 1.5;
		topShadowColor.green = (background.green > 65535/1.5) ?
			65535 : background.green * 1.5;
		topShadowColor.blue = (background.blue > 65535/1.5) ?
			65535 : background.blue * 1.5;

		bottomShadowColor.red = background.red * 0.5;
		bottomShadowColor.green = background.green * 0.5;
		bottomShadowColor.blue = background.blue * 0.5;
	
		/* 
		 * Lets start it with uppercase if we received uppercase 
		 */
		isUpper = isupper (*resStart);
		motifName = isUpper ? "Foreground: " : "foreground: ";

		/* If  the resource is multy - line we are out if luck */
		substitute (map, line, resStart - line, motifName, &foreground);

		motifName = isUpper ? "ArmColor: " : "armColor: ";
		substitute (map, line, resStart - line, motifName, &armColor);

		motifName = isUpper ? "TopShadowColor: " : "topShadowColor: "; 
		substitute (map, line, resStart - line, motifName, &topShadowColor);
		motifName = isUpper ? "BottomShadowColor: " : 
			"bottomShadowColor: "; 
		substitute (map, line, resStart - line, motifName, &bottomShadowColor);
	}
	return 1;
}

/*------------------------------------------------------------------------------
 * Function     : substitute
 * Description  : substitute a resource 
 * Returns      :
 *----------------------------------------------------------------------------*/
static void
substitute (colorMap *map, char *segment, int segmentSize, char *newResource, XColor *desiredColor )
{
	char	colorName[COLOR_SIZE];
	XColor	exact, use, xout;

	sprintf (colorName, "#%04x%04x%04x", 
		(int)desiredColor->red,
		(int)desiredColor->green,
		(int)desiredColor->blue);

	if (!grabColor (map, colorName, &exact, &use, &xout))
	{
		return;
	}

	fwrite (segment, segmentSize, 1, stdout);
	fprintf (stdout, "%s#%04x%04x%04x\n", 
		newResource,
		(int)use.red,(int)use.green,(int)use.blue);

	return;
}

/*------------------------------------------------------------------------------
 * Function     : rgb2HSV
 * Description  : convert rgb colors to HSV from lesstif
 * Returns      :
 *----------------------------------------------------------------------------*/
static void
rgb2HSV (unsigned short r, unsigned short g, unsigned short b, double *hr, double *sr, double *vr) 
{
	double rd, gd, bd, max, min, del;
	double rc, gc, bc;
	double v, h, s;

	/* convert RGB to HSV */
	rd = r / 65535.0;            /* rd,gd,bd range 0-1 instead of 0-65535 */
	gd = g / 65535.0;
	bd = b / 65535.0;
    
	/* compute maximum of rd,gd,bd */
	if (rd>=gd) { if (rd>=bd) max = rd;  else max = bd; }
	else { if (gd>=bd) max = gd;  else max = bd; }

	/* compute minimum of rd,gd,bd */
	if (rd<=gd) { if (rd<=bd) min = rd;  else min = bd; }
	else { if (gd<=bd) min = gd;  else min = bd; }


	del = max - min;
	v = max;
	if (max != 0.0) s = (del) / max;
	else s = 0.0;

	h = -1;
	if (s != 0.0) 
	{
		rc = (max - rd) / del;
		gc = (max - gd) / del;
		bc = (max - bd) / del;

		if      (rd==max) h = bc - gc;
		else if (gd==max) h = 2 + rc - bc;
		else if (bd==max) h = 4 + gc - rc;

		h = h * 60;
		if (h<0) h += 360;
	}

	*hr = h;  *sr = s;  *vr = v;
}
